#! /usr/bin/perl



# Flaristo: text search and report tool.
# This module implements the search engine.
# Copyright (C) 2001, 2002 Leandro Bernsmller
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

# Sintax: flsearch PARAMETERS WHERE-SEARCH

# PARAMETERS:

# -e EXPRESSION
# Regular EXPRESSION to search. This is mandatory.

# -w
# Convert HTML quotes to ASCII chars.

# -r
# Recursive search inside sub-directories.
# Don't follow simbolic links.

# -i
# Don't differ char case.

# -mN
# What match with the given EXPRESSION, N can be: 1, 2 or 3:
#   -m1: file name,
#   -m2: file contents (default),
#   -m3: file name and contents.

# -x SUFFIX COMMAND
# Define what COMMAND to use when accessing one file with the given
# SUFFIX. Used to preprocess the file before search its contents.
# %n represents the file name to be accessed.
# %e represents the file to be created temporarely (is %n without suffix).

# -o FILE
# Output result to the given FILE. Default is stdout.

# -a MASK
# Files to avoid in search.

# -T
# From the files to search, only search inside text files.

# -t DIR
# DIR is a directory to be used as temporary directory
# to store processing data. The default temp diretory
# is the one defined in tmp enviroment variable, if none,
# /tmp/ .

# -v
# Verbose mode, show more information about what is happening
# during execution.

# -c
# Output, before a line, one code acording its type.
# (Usefull to easy output process.)

# --version
# Show program version and exit.

# --help
# Show some help information and exit.

# WHERE-SEARCH:

# -s ENTRIES ...
# ENTRIES are places where search start. Default is the current directory.
# If some ENTRIES are files, search inside them. If ENTRIES have "-"
# search in stdin. This option must be the last.



# global variables
$prog_cmdn="flsearch";
$prog_name="Flaristo Search Engine";
$prog_version="0.1.4";
$prog_autor="Leandro Bernsmller";

%options_tab;
%extensions_tab;
@avoid_names;
@start_fs_entries;
$system_temp_dir = $ENV{TMP};
$base_temp_place = "";
$output_name = "-";
$search_expression = "";
$parse_parameters_error = "";
$preprocess_counter = 0;
@where_search = "";
$code_scan_dir = "(0) ";
$code_entry = "(1) ";
$code_searching = "(2) ";
$code_out_name = "(3) ";
$code_result_name = "(4) ";
$code_out_contents = "(5) ";
$code_result_contents = "(6) ";
$code_preprocess = "(7) ";
$code_cmd_preprocess = "(8) ";
$code_abort = "(9) ";
$code_unrecognized_option = "(a) ";
$code_no_search_expression = "(b) ";
$code_more_information = "(c) ";
$code_out_where_found = "(d) ";
$code_where_found = "(e) ";
$code_warn = "(f) ";



# subrotines
sub abort_process;
sub parse_command_line;
sub process_information_requests;
sub test_parameters;
sub init_scan;
sub end_scan;
sub scan_fs_entries;
sub scan_entry;
sub search_file_dir;
sub search_string;
sub remove_html_quotes;
sub preprocess_file;
sub output_result;



# Abort program execution. Before, send the given
# message to the given stream.
# Parameters list:
# - stream
# - message
sub abort_process {

  my $out_stream = $_[0];
  my $out_messag = $_[1];

  print $out_stream "$code_abort";
  print $out_stream $out_messag;

  exit;
}



# Warn user about process unespected results.
# Send a message to the given stream.
# Parameters list:
# - stream
# - message
sub warn_process {

  my $out_stream = $_[0];
  my $out_messag = $_[1];

  print $out_stream "$code_warn";
  print $out_stream "$out_messag\n";
}



# Analise command line parameters. Set global variables
# acording the given parameters.
sub parse_command_line {
  
  # init operation table
  $options_tab{"verbose"} = 0;
  $options_tab{"html_convert"} = 0;
  $options_tab{"recursive-subdirs"} = 0;
  $options_tab{"ignore-char-case"} = 0;
  $options_tab{"no-file-contents"} = 0;
  $options_tab{"line-code"} = 0;
  $options_tab{"text-only"} = 0;
  $options_tab{"version"} = 0;
  $options_tab{"help"} = 0;
  $options_tab{"what-match"} = 2;  # what compare
  $parse_parameters_error="";
  
  my $end = 0;
  while ($end < 3) {

    my $arg = shift @ARGV;

    if ( $arg eq "" ) {
      $end = 3;
    }

    PARAM_SWITCH: {
      if ($end > 0) {
	  last PARAM_SWITCH;
      }

      if ($arg eq "-v") {
	$options_tab{"verbose"} = 1;
	last PARAM_SWITCH;
      }

      if ($arg eq "-w") {
	$options_tab{"html-convert"} = 1;
	last PARAM_SWITCH;
      }

      if ($arg eq "-r") {
	$options_tab{"recursive-subdirs"} = 1;
	last PARAM_SWITCH;
      }

      if ($arg eq "-i") {
	$options_tab{"ignore-char-case"} = 1;
	last PARAM_SWITCH;
      }

      if ($arg eq "-c") {
	$options_tab{"line-code"} = 1;
	last PARAM_SWITCH;
      }

      if ($arg eq "-T") {
	$options_tab{"text-only"} = 1;
	last PARAM_SWITCH;
      }

      if ($arg eq "-m1") {
	$options_tab{"what-match"} = 1;
	last PARAM_SWITCH;
      }

      if ($arg eq "-m2") {
	$options_tab{"what-match"} = 2;
	last PARAM_SWITCH;
      }

      if ($arg eq "-m3") {
	$options_tab{"what-match"} = 3;
	last PARAM_SWITCH;
      }

      if ($arg eq "--version") {
	$options_tab{"version"} = 1;
	last PARAM_SWITCH;
      }

      if ($arg eq "--help") {
	$options_tab{"help"} = 1;
	last PARAM_SWITCH;
      }

      if ($arg eq "-x") {
	# register file name extension, add application
	my $ex = shift @ARGV;
	$extensions_tab{$ex} = shift @ARGV;
	last PARAM_SWITCH;
      }

      if ($arg eq "-o") {
	# get output file name
	$output_name = shift @ARGV;
	last PARAM_SWITCH;
      }

      if ($arg eq "-a") {
	# register files to avoid in search
	push @avoid_names, shift @ARGV;
	last PARAM_SWITCH;
      }

      if ($arg eq "-s") {
	# to get start directories, change parsing sheme
	$end = 1;
	last PARAM_SWITCH;
      }

      if ($arg eq "-t") {
	# get temp directory
	$temp_dir = shift @ARGV;
	last PARAM_SWITCH;
      }

      if ($arg eq "-e") {
	# get search expression
	$search_expression = shift @ARGV;
	last PARAM_SWITCH;
      }

      if ($end == 0) {
	# error, option not recognized
	$parse_parameters_error=$arg, if $arg ne "";
      }
    }

    if ($end == 2) {
	push @start_fs_entries, $arg;
      }

    if ($end == 1) {
      $end = 2;
    }
  }

  # if no start entries are defined, defines the work directory
  if ($#start_fs_entries < 0) {
    push @start_fs_entries, ".";
  }
}



# Show basic program information.
# If is the case, stop execution.
sub process_information_requests {

  if ( $options_tab{"version"} == 1) {
    print "$prog_cmdn $prog_version\n";
    print "Copyright 2001, 2002 $prog_autor, license: GNU-GPL 2.0\n\n";
    exit 0;
  }

  if ( $options_tab{"help"} == 1) {
    print "$prog_name $prog_version, copyright 2001, 2002 $prog_autor\n";
    print "Sintax: $prog_cmdn PARAMETERS WHERE-SEARCH\n";
    print "PARAMETERS:\n";
    print " -e EXPRESSION  Regular EXPRESSION to search (_mandatory_ option).\n";
    print " -w         Convert HTML quotes, in match contents, to ASCII chars.\n";
    print " -r         Recursive search inside sub-directories.\n";
    print "            Don't follow simbolic links.\n";
    print " -i         Ignore case - don't differ char case.\n";
    print " -mN        What match with the given EXPRESSION (N can be: 1, 2 or 3):\n";
    print "            -m1: file name, -m2: file contents (default),\n";
    print "            -m3: file name and contents.\n";
    print " -x SUFFIX COMMAND  Define the preprocess COMMAND for files with\n";
    print "                    the given SUFFIX. %n would be replaced by the file name.\n";
    print "                    %e would be replaced by file name (%n) without suffix.\n";
    print " -o FILE     Output file with the result, default: stdout.\n";
    print " -a MASK     Files mask indicating what avoid in search.\n";
    print " -T          From the files to search, only search contents of text files.\n";
    print " -v          Verbose mode.\n";
    print " -t DIR      DIR is the temp directory; the default is the one defined\n";
    print "             in enviroment variable TMP, or, if none, /tmp/ .\n";
    print " -c          Output, before a line, one code acording its type.\n";
    print " --version   Show program version and exit.\n";
    print " --help      Show some help information and exit.\n";
    print "WHERE-SEARCH - defining where start search:\n";
    print " -s ENTRIES  ENTRIES are places where start search;\n";
    print "             if any of -s and ENTRIES are omited,\n";
    print "             the current work directory is used as default. If ENTRIES have\n";
    print "             file names, search inside the correspondent files. If any entry\n";
    print "             in ENTRIES is -, get search data from stdin.\n";
    print "             This option _must_ be the last.\n";
    exit 0;
  }
}


# Verify if need append one code for each line type in output.
# If don't, remove the codes.
sub disable_codes {
  if ( $options_tab{"line-code"} == 0 ) {

    $code_scan_dir = "";
    $code_entry = "";
    $code_searching = "";
    $code_out_name = "";
    $code_result_name = "";
    $code_out_contents = "";
    $code_result_contents = "";
    $code_preprocess = "";
    $code_cmd_preprocess = "";
    $code_abort = "";
    $code_unrecognized_option = "";
    $code_no_search_expression = "";
    $code_more_information = "";
    $code_out_where_found = "";
    $code_where_found = "";
    $code_warn = "";
  }
}



# Test if all needed data was right defined
# in command line parameters. If has errors
# show error messages and exit with code 1.
sub test_parameters {

  my $error = 0;

 ERROR_BLOCK: {

    if ($parse_parameters_error ne "") {
      print "$code_unrecognized_option";
      print "Unrecognized option: $parse_parameters_error\n";
      $error = 1;
      last ERROR_BLOCK;
    }

    if ($search_expression eq "") {
      print "$code_no_search_expression";
      print "No search expression defined.\n";
      $error = 1;
      last ERROR_BLOCK;
    }
  }

  if ($error == 1) {
    print "$code_more_information";
    print "For more information type: $prog_cmdn --help\n";
    exit 1;
  }
}



# Prepare to scan file system.
sub init_scan {

  # try open the output stream
  open outfh, ">$output_name";

  # define name for temp directory

  if ($system_temp_dir eq "") {  # no shell enviroment definition for tmp variable
    $system_temp_dir = "/tmp";
  }

  # name for directory to be created inside temp dir, if necessary
  $base_temp_place = join ( "", $system_temp_dir, "/$prog_cmdn.", $$ );
}  



# Finish scan process.
sub end_scan {

  close outfh;

  rmdir $base_temp_place if -e $base_temp_place;
}  



# Make search for each one of the given entries.
sub scan_fs_entries {

  foreach $en (@start_fs_entries) {
    if ($en ne "") {
      scan_entry "", $en, $search_expression;
    }
  }
}



# Make search on the given directory.
# Parameters List:
# - path to be removed from entry, defined at preprocess time
# - start file system entry to make search
# - regular expression to search for
sub scan_entry {

  my $remove_path = $_[0];  # normaly the temp related path
  my $start_entry = $_[1];
  my $expression = $_[2];

  # define mask to glob entry names
  if ( ! -f $start_entry ) {  # entry is not a file

    # define files mark
    if ($start_entry eq "") { # entry is not defined
      $scan_mask = "*";
    }
    else {                    # supose entry is a directory
      $scan_mask = join ("", $start_entry, "/*");
    }
  }
  else {                      # supose entry is a file
    $scan_mask = $start_entry;
  }

  # show information, if in verbose mode
  if ( $options_tab{"verbose"} == 1 &&
     -d $start_entry && ! -l $start_entry &&
       $preprocess_counter == 0 ) {  # don't output tmp dir scanning
    print outfh "$code_scan_dir";
    print outfh "Pesquisando no diretrio: $start_entry\n";
  }

  # for each entry in directory
  foreach $entry (glob "$scan_mask") {

    # if using tmp dir (for preprocessing and scan)
    my $light_entry = $entry;
    # remove tmp path from entry string to output and name search
    if ($preprocess_counter >= 1) {
      $light_entry =~ s/$remove_path\///;
    } else {
      $light_entry =~ s/$remove_path\//\//;
    }

    # verify if must avoid search inside this entry
    my $skip_entry = 0;
    my $tmp_entry = $entry;
    foreach $pattern (@avoid_names) {
      my $avoid_pattern = quotemeta $pattern;
      my $res = ( $tmp_entry =~ m/$avoid_pattern/ );
      if ($res ne "") {
	$skip_entry = 1;
      }
    }

    # process entry, if is the case
    if ($skip_entry == 0) {

      # if entry is a directory to scan
      if ( -d $entry &&
	 ! -l $entry ) {  # don't follow symbolic links

	# show information
        print outfh "$code_entry";
	print outfh "$light_entry\n";

	search_file_dir $light_entry, $entry, $expression, 1;

	# scan inside subdir, if is the case
	if ($options_tab{"recursive-subdirs"} == 1 ) {
	  scan_entry $remove_path, $entry, $expression;
	}
      }

      # if entry is a file
      if ( -f $entry ) {

	# show information
        print outfh "$code_entry";
	print outfh "$light_entry\n";

	search_file_dir $light_entry, $entry, $expression, 0;
      }
    }
  }
}

# Output the result of one search match.
# Parameters List:
# - search result
# - file name entry without tmp path (added for preprocess)
sub output_result {

  my $result = $_[0];  
  my $light_entry = $_[1];

	print outfh "$code_out_contents";
	print outfh "--- foi encontrado o contedo:\n";
	print outfh "$code_result_contents";
	print outfh "$result\n";
	print outfh "$code_out_where_found";
	print outfh "--- em:\n";

  foreach $entry (@where_search) {
	  if ($entry ne "") {
			print outfh "$code_where_found";
			print outfh "$entry\n";
		}
	}
	print outfh "$code_where_found";
	print outfh "$light_entry\n";
}



# Search for the given regular expression in
# the given file or directory name.
# Parameters List:
# - file name entry without tmp path (added for preprocess)
# - file name entry (complete, with tmp path, if is the case)
# - search expression
# - if need test only entries names: 1-only name; 0-contents too
sub search_file_dir {

  my $light_entry = $_[0];
  my $entry = $_[1];  
  my $expression = $_[2];
  my $search_name_only = $_[3];

  $out_header="Pesquisando...\n";

  # if need to search in file/dir name
  if ($options_tab{"what-match"}==1 ||
      $options_tab{"what-match"}==3) {

    # show information, if in verbose mode
    if ( $options_tab{"verbose"} == 1 ) {
      #print outfh "$code_searching";
      #print outfh $out_header;
    }

    # extract path from light entry
    my $name_only = $light_entry;
    $name_only =~ s/.*\///i;

    # search in file name
    my $search_result = search_string $name_only, $expression;

    # verify if was found something
    if ($search_result ne "") {
      output_result $search_result, $light_entry;
    }
  }

  # if matching only name, don't continue
  if ($search_name_only == 1) {
    return;
  }

  # try preprocess the file
  my $prep = preprocess_file $light_entry, $entry;
  if ( $prep == 1 ) {  # if preprocess, don't search inside
    return;
  }

  # if need to search inside file
  if ($options_tab{"what-match"}==2 ||
      $options_tab{"what-match"}==3) {

    # verify if need search only inside text files
    if ($options_tab{"text-only"}==1 &&
	! -T $entry) {  # this is not a text file
      return;
    }

    # show information, if in verbose mode
    if ( $options_tab{"verbose"} == 1 &&
	 $options_tab{"what-match"}!=3) { # don't wrote this previouly
      #print outfh "$code_searching";
      #print outfh $out_header;
    }

    # open file to get contents
    open (searchfh, $entry);
    # read file
    my $file_size = -s searchfh;
    my $file_contents;
    my $num_read = read searchfh, $file_contents, $file_size;
    chop $file_contents;

    # search inside file
    my $search_result = search_string $file_contents, $expression;

    # verify if was found something
    if ($search_result ne "") {
      output_result $search_result, $light_entry;
    }

    # search inside file
    #while (<searchfh>) {
    #  m/$expression/mig;
    #  print "[$_]\n";
    #  if ($& ne "") {
    #    print " encontrou $& em $_\n";
    #  }
    #}

    # close file
    close searchfh;
  }
}



# Search for the given regular expression in
# the given string scalar.
# Parameters List:
# - string
# - expression
# Return:
# the pattern found or empty string if nothing found.
sub search_string {

  my $str = $_[0];
  my $expression = $_[1];

  # verify if user tells to remove html quotes (converting them)
  if ($options_tab{"html-convert"} == 1) {
    $str = remove_html_quotes $str;
  }

  # perform search in string
  my $res;  # search result
  # verify if user tells to ignore char cases
  if ($options_tab{"ignore-char-case"} == 1) {
    # if something was found
    if ( $str =~ m/$expression/mgi ) {
      $res = $&;
    } else {
      $res = "";
    }
  } else {
    # if something was found
    if ( $str =~ m/$expression/mg ) {
      $res = $&;
    } else {
      $res = "";
    }
  }

  return $res;
}



# Remove html quotes from the given text, return the
# converted text.
# Parameters List:
# - the text (string scalar)
sub remove_html_quotes {

  my $text = $_[0];

  $text =~ s/&atilde;//;
  $text =~ s/&aacute;//;
  $text =~ s/&agrave;//;
  $text =~ s/&acirc;//;
  $text =~ s/&auml;//;
  $text =~ s/&eacute;//;
  $text =~ s/&ecirc;//;
  $text =~ s/&iacute;//;
  $text =~ s/&otilde;//;
  $text =~ s/&oacute;//;
  $text =~ s/&ocirc;//;
  $text =~ s/&ouml;//;
  $text =~ s/&uacute;//;
  $text =~ s/&uuml;//;
  $text =~ s/&ccedil;//;
  
  $text =~ s/&Atilde;//;
  $text =~ s/&Aacute;//;
  $text =~ s/&Agrave;//;
  $text =~ s/&Acirc;//;
  $text =~ s/&Auml;//;
  $text =~ s/&Eacute;//;
  $text =~ s/&Ecirc;//;
  $text =~ s/&Iacute;//;
  $text =~ s/&Otilde;//;
  $text =~ s/&Oacute;//;
  $text =~ s/&Ocirc;//;
  $text =~ s/&Ouml;//;
  $text =~ s/&Uacute;//;
  $text =~ s/&Uuml;//;
  $text =~ s/&Ccedil;//;

  return $text;
}



# Preprocess the given file, if need.
# Parameters list:
# - file name without tmp path
# - file name
# return:
# 1 - file preprocessed / 0 - don't / 2 - error
sub preprocess_file {

  my $light_entry = $_[0];
  my $entry = $_[1];

  # verify if need preprocess file

  my $need_pre = 0;

  # get extension from entry name
  my $entry_noext = $entry;
  $entry_noext =~ s/\..([^.]*)$//gi; # get path and file name without extension
  my $entry_ext = $&;    # supose it is between last "." and text end
  # compare keys, verify if entry extension match any extension to preprocess
  foreach $e (keys %extensions_tab) {
    if ( $e eq $entry_ext ) {
      $need_pre = 1;
    }
  }

  # if don't need preprocess, extension don't match
  if ($need_pre == 0) {
    return 0;
  }

  # preprocess

  if ( $options_tab{"verbose"} == 1 ) {
    print outfh "$code_preprocess";
    print outfh "Preprocessando...\n";
  }

  # define command
  my $cmd_preproc = $extensions_tab{$entry_ext};

  # define current work directory
  my $work_dir = `pwd` or abort_process (outfh, "No foi possvel determinar o diretrio que est em uso atualmente.");
  chop $work_dir;

  # try define entry absolute location (need REVIEW)
  # supose entry is in work directory
  my $entry_place = join ("", $work_dir, "/", $entry);
  if (! -e $entry_place) {
    # if can't find it, supose it's path is absolute
    $entry_place = $entry;  
  }

  # add entry name to command, replace symbols (%n and %e) by file names
  $entry_noext =~ s/.*\///i;  # remove path from file name
  $cmd_preproc =~ s/%n/$entry_place/;
  $cmd_preproc =~ s/%e/$entry_noext/;

  # create temp subdirectories, if need
  if (! -e $base_temp_place) {
    mkdir $base_temp_place, umask;
  }
  my $temp_place = join ("", $base_temp_place, "/", $preprocess_counter);
  if (! -e $temp_place) {
    mkdir $temp_place, umask;
  }

  # change work directory to temp directory
  chdir $temp_place or abort_process (outfh, "O diretrio temporrio no foi encontrado.");

  # execute command
  if ( $options_tab{"verbose"} == 1 ) {
    print outfh "$code_cmd_preprocess";
    print outfh "$cmd_preproc\n";
  }
  `$cmd_preproc` or warn_process (outfh, "O preprocessamento falhou inesperadamente.");

  # update preprocess counter
  $preprocess_counter ++;

  # goto previous work directory
  chdir $work_dir or abort_process (outfh, "O diretrio de origem no foi encontrado.");

  # push the preprocessed file in the list of places gone to make search
  push @where_search, $light_entry;

  # scan inside preprocess result
  scan_entry $temp_place, $temp_place, $search_expression;

  # pop the preprocessed file from the list of places gone to make search
  pop @where_search;

  # delete preprocessed data (REVIEW)
  `rm -Rf $temp_place`;
  
  return $need_pre;
}	
	
	
	
parse_command_line;
process_information_requests;
disable_codes;
test_parameters;
init_scan;
scan_fs_entries;
end_scan;	
